// defines
#define SND PB0  // sound sensor
#define BUT PD2  // on-off button
#define LED PD4  // on-off led
#define SVD PB1  // servo down
#define SVU PB2  // servo up
#define ELR PD5  // eye led right
#define ELL PD6  // eye led left

// globals
volatile char blink;
volatile char dance;
volatile char sleep;
char servo_dir = 0;
char blink_dir = 0;

void setup() {
  // init globals
  blink = 0;
  dance = 0;
  sleep = 1;
  
  // gpio stuff
  DDRB &= ~(1 << SND);    // set sound to input
  PORTB |= (1 << SND);    // pull up

  DDRD &= ~(1 << BUT);    // set button port to input
  PORTD |= (1 << BUT);    // button pull-up
  
  DDRD |= (1 << LED);     // set led port to output
  PORTD &= ~(1 << LED);   // led is off initially
  
  DDRB |= (1 << SVD);     // set servo down port output
  DDRB |= (1 << SVU);     // set servo up port output

  DDRD |= (1 << ELR);     // set eye led right output
  DDRD |= (1 << ELL);     // set eye led left output

  // sound interrupt
  PCICR = 0;              // make sure pcint0 is disabled now
  
  // button interrupts - always on
  EICRA |= (2 << ISC00);  // interrupt on high to low
  EIMSK |= (1 << INT0);   // activate int0
  
  sei();                  // activate global

  // configure timer 2 - tick overflow
  TIMSK2 = 0;               // disable timer 2 interrupts for now
  TCCR2A = 0;               // set mode normal ovf
  TCCR2B = 0;
  TCCR2B |= (7 << CS20);    // prescaler 1024
  
  // configure timer 1 - pwm
  TIMSK1 = 0;              // make sure timer 1 interrupts are off
  
  TCCR1A = 0;
  TCCR1A |= (1 << COM1A1); // non inverting
  TCCR1A |= (1 << COM1B1); // non inverting
  TCCR1A |= (2 << WGM10);  // set mode fast pwm
  
  TCCR1B = 0;
  TCCR1B |= (3 << WGM12);  // also fast pwm - but like with ICR as top
  TCCR1B |= (3 << CS10);   // prescaler 64
  
  TCNT1 = 0;               // initialize count
  ICR1 = 0x1388;           // send signal with 50Hz
  OCR1A = 316;
  OCR1B = 316;
  
  // configure timer 0 - pwm 8 bit - top 0xFF
  TCCR0A = 0;
  TCCR0A |= (1 << COM0A1); // non-inverting
  TCCR0A |= (1 << COM0B1); // non-inverting
  TCCR0A |= (3 << WGM00);  // fast pwm
  
  TCCR0B = 0;
  TCCR0B |= (5 << CS00);   // prescaler 1024

  TCNT0 = 0;               // initialize count
  OCR0A = 0;
  OCR0B = 0xFF / 2;
  
  // enable sleep
  SMCR |= (2 << SM0);      // set mode power down
  SMCR |= (1 << SE);       // enable sleep
}

// button interrupt
ISR(INT0_vect) {
  if (!(PIND & (1 << LED))) { // if led off
    
    PORTD ^= (1 << LED);      // toggle led
    OCR0A = 0;
    OCR0B = 0xFF / 2;
    
    PCIFR |= (1 << PCIF0);    // clear flag
    PCMSK0 |= (1 << PCINT0);
    PCICR |= (1 << PCIE0);    // enable listening
  } else {                    // if led off
    PCIFR |= (1 << PCIF0);    // stop pcint0
    PCMSK0 &= ~(1 << PCINT0);
    PCICR &= ~(1 << PCIE0);
    
    // stop everything
    blink = 0;
    dance = 0;
    TIFR2 |= (1 << OCF2A);    // clear flag - write logic one
    TIMSK2 &= ~(1 << TOIE2);  // disable timer 2
    
    PORTD ^= (1 << LED);
    OCR0A = 0;
    OCR0B = 0;
    
    sleep = 1;                // go to sleep
    SMCR |= (1 << SE);        // enable sleep
  }

  EIFR |= (1 << INT0);        // clear int0 flag
  delay(10);
}

// sound interrupt - on PB0 of PCINT0
ISR(PCINT0_vect) {
  if ((PINB & (1 << PB0)) == 0) {
    dance = 1;               // enable dancing

    PCIFR |= (1 << PCIF0);   // stop pcint0
    PCMSK0 &= ~(1 << PCINT0);
    PCICR ^= (1 << PCIE0);   // don't listen while dancing

    blink = 0;
    TIFR2 |= (1 << OCF2A);   // clear flag - write logic one
    TCNT2 = 0;               // start counting now
    TIMSK2 |= (1 << TOIE2);  // enable timer 2
  }
}

// timer - tick max overflow - stop dancing
ISR(TIMER2_OVF_vect) {
  blink++;

  if (blink == 100) {
    blink = 0;
  
    dance = 0;                // disable dancing
  
    TIFR2 |= (1 << OCF2A);    // clear flag
    TIMSK2 &= ~(1 << TOIE2);  // disable timer 2
  
    PCIFR |= (1 << PCIF0);    // clear pcint0 flag before enable
    PCMSK0 |= (1 << PCINT0);
    PCICR |= (1 << PCIE0);    // enable listening again
  }
}

void loop() {
  // sleep stuff
  if (sleep) {
    asm("SLEEP");  // power down
    sleep = 0;     // if we reach this we woke up
  }
  
  // servos move in different directions
  if (dance) {
    // move servos
    if (servo_dir) {
      OCR1A += 5;
      OCR1B -= 5;
      if (OCR1A >= 435) servo_dir = 0;
    } else {
      OCR1A -= 5;
      OCR1B += 5;
      if (OCR1A <= 197) servo_dir = 1;
    }

    // blink leds
    if (blink_dir) {
      OCR0A++;
      OCR0B--;
      if (OCR0A == 0xFF / 2) blink_dir = 0;
    } else {
      OCR0A--;
      OCR0B++;
      if (OCR0A == 0) blink_dir = 1;
    }
  }
  
  delay(2);
}
